/*
 *   libaudiotag - A media file tag-reader library
 *   Copyright (C) 2003, 2004  Pipian
 *
 *   This library is free software; you can redistribute it and/or
 *   modify it under the terms of the GNU Lesser General Public
 *   License as published by the Free Software Foundation; either
 *   version 2.1 of the License, or (at your option) any later version.
 *
 *   This library is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 *   Lesser General Public License for more details.
 *
 *   You should have received a copy of the GNU Lesser General Public
 *   License along with this library; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 *
 */
 
#include <stdio.h>
#include <stdlib.h>
#include "cdaudio.h"
#include "id3v1.h"
#include "id3v2.h"
#include "ape.h"
#include "vorbis.h"
#include "itunes.h"
#include "main.h"
#include "../fmt.h"

char *tag_exists(char *filename)
{
	FILE *fp;
	int status = 0;
	static char tags[2];

	tags[0] = NO_TAGS;
	tags[1] = NO_TAGS;

	/* Check for CD Audio */
	if(findCDAudio(filename))
	{
		pdebug("Found CD Audio...");
		tags[0] = CD_AUDIO;

		return tags;
	}

	fp = fopen(filename, "r");
	if(fp == NULL)
	{
		pdebug("Error reading file...");
		tags[0] = READ_ERROR;

		return tags;
	}

	/* Check for ID3v1 */
	fseek(fp, -128, SEEK_END);
	if(findID3v1(fp))
	{
		pdebug("Found ID3v1 tag...");
		tags[1] |= ID3V1_TAG;
	}

	/* Check for ID3v2 */
	fseek(fp, 0, SEEK_SET);
	status = findID3v2(fp);
	if(status > -1)
	{
		pdebug("Found ID3v2 tag...");
		tags[1] |= ID3V2_TAG;
	}
	status = 0;

	/* Check for Ogg Vorbis */
	fseek(fp, 0, SEEK_SET);
	status = findVorbis(fp);
	if(status > -1)
	{
		pdebug("Found Vorbis comment block...");
		tags[1] |= VORBIS_TAG;
	}
	status = 0;

	/* Check for FLAC */
	fseek(fp, 0, SEEK_SET);
	if(findFlac(fp))
	{
		pdebug("Found FLAC tag...");
		tags[1] |= FLAC_TAG;
	}
	
	/* Check for OggFLAC */
	fseek(fp, 0, SEEK_SET);
	status = findOggFlac(fp);
	if(status > -1)
	{
		pdebug("Found OggFLAC...");
		tags[1] |= OGGFLAC_TAG;
	}
	status = 0;

	/* Check for Speex */
	fseek(fp, 0, SEEK_SET);
	status = findSpeex(fp);
	if(status > -1)
	{
		pdebug("Found Speex...");
		tags[1] |= SPEEX_TAG;
	}
	status = 0;

	/* Check for APE */
	fseek(fp, 0, SEEK_SET);
	status = findAPE(fp);
	if(status > 0)
	{
		pdebug("Found APE tag...");
		tags[1] |= APE_TAG;
	}
	status = 0;
	
	/* Check for iTunes M4A */
	fseek(fp, 4, SEEK_SET);
	status = findiTunes(fp);
	if(status > -1)
	{
		pdebug("Found iTunes tag...");
		tags[1] |= ITUNES_TAG;
	}

	fclose(fp);

	return tags;
}
